@@ -401,27 +401,15 @@ def install(args, parser, command='install'):
     # for 'conda update', make sure the requested specs actually exist in the prefix
     # and that they are name-only specs
     if isupdate and context.update_modifier == UpdateModifier.UPDATE_ALL:
-        history_dict = History(prefix).get_requested_specs_map()
-        pins = {
-            pin.name: pin
-            for pin in get_pinned_specs(prefix)
-        }
-        # for key, match_spec in history_dict.items():
-        for key in installed_names:
-            if key == 'python':
-                i = installed_names.index('python')
-                version = installed_pkg_recs[i].version
-                py_ver = ".".join(version.split(".")[:2]) + '.*'
-                # specs.append(MatchSpec(name="python", version=py_ver))
-            else:
-                if key in pins:
-                    specs.append(pins[key])
-                else:
-                    specs.append(MatchSpec(key))
+        for i in installed_names:
+            if i != 'python':
+                specs.append(MatchSpec(i))
 
         prefix_data = PrefixData(prefix)
         for s in args_packages:
             s = MatchSpec(s)
+            if s.name == 'python':
+                specs.append(s)
             if not s.is_name_only_spec:
                 raise CondaValueError("Invalid spec for 'conda update': %s\n"
                                       "Use 'conda install' instead." % s)
@@ -503,13 +491,28 @@ def install(args, parser, command='install'):
     )
     solver.add_jobs(mamba_solve_specs, solver_task)
 
-    # as a security feature this will _always_ attempt to upgrade certain packages
-    for a_pkg in [_.name for _ in context.aggressive_update_packages]:
-        if a_pkg in installed_names:
-            solver.add_jobs([a_pkg], api.SOLVER_UPDATE)
+    if not context.force_reinstall:
+        # as a security feature this will _always_ attempt to upgrade certain packages
+        for a_pkg in [_.name for _ in context.aggressive_update_packages]:
+            if a_pkg in installed_names:
+                solver.add_jobs([a_pkg], api.SOLVER_UPDATE)
 
     if python_constraint:
-        solver.add_constraint(python_constraint)
+        solver.add_pin(python_constraint)
+
+    pinned_specs = get_pinned_specs(context.target_prefix)
+    if pinned_specs:
+        conda_prefix_data = PrefixData(context.target_prefix)
+    for s in pinned_specs:
+        x = conda_prefix_data.query(s.name)
+        if x:
+            for el in x:
+                if not s.match(el):
+                    print("Your pinning does not match what's currently installed. Please remove the pin and fix your installation")
+                    print("  Pin: {}".format(s))
+                    print("  Currently installed: {}".format(el))
+                    exit(1)
+        solver.add_pin(str(s))
 
     success = solver.solve()
     if not success:
@@ -576,12 +579,15 @@ def update(args, parser):
     install(args, parser, 'update')
 
 def repoquery(args, parser):
-
     if not args.subcmd:
         print("repoquery needs a subcommand (search, depends or whoneeds)")
         print("eg:")
         print("    $ mamba repoquery search xtensor\n")
         exit(1)
+
+    if args.platform:
+        context._subdirs = (args.platform, 'noarch')
+
     prefix = context.target_prefix
 
     init_api_context()
@@ -599,17 +605,25 @@ def repoquery(args, parser):
     pool = api.Pool()
     repos = []
 
-    # add installed
-    repo = api.Repo(pool, "installed", installed_json_f.name, "")
-    repo.set_installed()
-    repos.append(repo)
-
     only_installed = True
+    channels = args.channel or []
     if args.subcmd == "search" and args.installed == False:
         only_installed = False
-    elif args.all_channels:
+    elif args.all_channels or len(channels):
         only_installed = False
 
+    if only_installed and args.no_installed:
+        print("No channels selected.")
+        print("Activate -a to search all channels.")
+        exit(1)
+
+    if not args.no_installed:
+        # add installed
+        repo = api.Repo(pool, "installed", installed_json_f.name, "")
+        repo.set_installed()
+        repos.append(repo)
+
+
     if not only_installed:
         index = get_index(channel_urls=index_args['channel_urls'],
                       prepend=index_args['prepend'], platform=None,
@@ -632,7 +646,7 @@ def repoquery(args, parser):
     if args.subcmd == "whoneeds":
         query.whoneeds(args.package_query, args.tree)
     if args.subcmd == "depends":
-        query.depends(args.package_query)
+        query.depends(args.package_query, args.tree)
     if args.subcmd == "search":
         query.find(args.package_query)
 
@@ -692,6 +706,9 @@ Examples:
         help=SUPPRESS
     )
 
+    package_cmds.add_argument('-p', '--platform')
+    package_cmds.add_argument('--no-installed', action='store_true')
+
     package_cmds.add_argument(
         "-a", "--all-channels",
         action="store_true",
@@ -704,26 +721,27 @@ Examples:
         action="store_true"
     )
 
-    subsub_parser.add_parser('whoneeds',
+    c1 = subsub_parser.add_parser('whoneeds',
         help='shows packages that depends on this package',
         parents=[package_cmds, view_cmds]
     )
 
-    subsub_parser.add_parser('depends',
+    c2 = subsub_parser.add_parser('depends',
         help='shows packages that depends on this package',
         parents=[package_cmds, view_cmds]
     )
 
-    subsub_parser.add_parser('search',
+    c3 = subsub_parser.add_parser('search',
         help='shows packages that depends on this package',
         parents=[package_cmds]
     )
 
     from conda.cli import conda_argparse
-    conda_argparse.add_parser_channels(p)
-    conda_argparse.add_parser_networking(p)
-    conda_argparse.add_parser_known(p)
-    conda_argparse.add_parser_json(p)
+    for cmd in (c1, c2, c3):
+        conda_argparse.add_parser_channels(cmd)
+        conda_argparse.add_parser_networking(cmd)
+        conda_argparse.add_parser_known(cmd)
+        conda_argparse.add_parser_json(cmd)
 
     p.set_defaults(func='.main_repoquery.execute')
     return p
@@ -752,7 +770,8 @@ def _wrapped_main(*args, **kwargs):
 
     init_loggers(context)
 
-    exit_code = do_call(args, p)
+    result = do_call(args, p)
+    exit_code = getattr(result, 'rc', result) # may be Result objects with code in rc field
     if isinstance(exit_code, int):
         return exit_code
 
